"""
Initiate logger and its all injection and format

SHOULD not using `logging.getLogger(__name__)` to get logger but using
`middlewares.logger.get_logger(__name__)` instead.
"""
import logging

from fastapi import logger as fastapi_logger

from .. import package_name

gunicorn_error_logger = logging.getLogger("gunicorn.error")


def init_app():
    """Init uvicorn and fastapi logger to unify log template with gunicorn"""
    uvicorn_access_logger = logging.getLogger("uvicorn.access")
    uvicorn_access_logger.addFilter(NameInjectionFilter())
    uvicorn_access_logger.handlers = gunicorn_error_logger.handlers
    fastapi_logger.handlers = gunicorn_error_logger.handlers


def get_logger(name: str):
    """Function to get logger"""
    logger = logging.getLogger(name)
    logger.handlers = gunicorn_error_logger.handlers
    logger.addFilter(NameInjectionFilter())
    return logger


class NameInjectionFilter(logging.Filter):
    """Add module name to log line. But shorten it like Logback"""

    def filter(self, record: logging.LogRecord):
        name = package_name.shorten(record.name, 35)
        record.msg = name + " | " + record.msg
        return True



from typing import Optional, Dict, Any

from fastapi import FastAPI, HTTPException, Request, status
from fastapi.encoders import jsonable_encoder
from fastapi.exceptions import RequestValidationError
from fastapi.responses import JSONResponse
from translation import google


def init_app(app: FastAPI):
    app.add_exception_handler(RequestValidationError, handle_validation_error)


def handle_validation_error(request: Request, exc: RequestValidationError):
    error_message = exc.errors()[0].get("msg")
    error = InvalidParam(msg=error_message)
    return error.to_response()


class ZOGException(HTTPException):
    """
    Default ZOG Exception

    Auto translate exception message to supported language via Google. Default
    support English. ZOG exception MUST BE init with error message in English.
    """
    status_code: status
    detail: dict = []
    supported_lang = ["vi"]

    def __init__(self, msg: str = None,
                 headers: Optional[Dict[str, Any]] = None):
        self.detail["en"] = msg
        self.add_detail_lang(msg)
        super(ZOGException, self).__init__(
            status_code=self.status_code,
            detail=self.detail,
            headers=headers
        )

    def add_detail_lang(self, msg: str):
        for lang in self.supported_lang:
            self.detail[lang] = google(msg, dst=lang)

    def to_response(self) -> JSONResponse:
        return JSONResponse(
            status_code=self.status_code,
            content=jsonable_encoder({
                "detail": self.detail
            })
        )


class InvalidParam(ZOGException):
    status_code = status.HTTP_400_BAD_REQUEST
    detail = {
        "en": "Invalid param",
        "vi": "Thiếu thông tin"
    }



def shorten(package_name: str, max_length: int, level: int = 1) -> str:
    """
    Shorten package name like Convention Word in logback - but in a simpler
    version.
    Eg: package.name.class.Function will be shorten to p.n.c.Function

    @param package_name: name of package to shorten
    @param max_length: max length of shortened name - shortened name will be
    smaller or equal this value. If smaller - add space.
    @param level: to use this function recursive, add level to travel over dots

    >>> assert shorten("thit.cho.mam.tom", 9) == "t.c.m.tom"
    >>> assert shorten("thit.cho.mam.tom", 11) == "t.c.mam.tom"
    >>> assert shorten("thit.cho.mam.tom", 10) == "t.c.m.tom "
    """
    package_len = len(package_name)
    if package_len < max_length:
        return package_name.ljust(max_length, " ")
    elif package_len == max_length:
        return package_name
    dots = package_name.count(".")

    if dots < level:
        return package_name

    names = package_name.split(".")
    names[level-1] = names[level-1][0]
    new_package_name = ".".join(names)
    return shorten(new_package_name, max_length, level+1)



import pydantic
from fastapi import FastAPI

from . import securities, monitoring, logger


def init_app(app: FastAPI, settings: pydantic.BaseSettings):
    logger.init_app()
    monitoring.init_app(app, settings)
    securities.init_app(app, settings)



"""
Security middlewares

Help project to enable security function faster and easy to config via settings
environment parameters.

Configs:
    - SECURITY_CSRF: True or False to enable CSRF
    - BACKEND_CORS_ORIGINS: List of CORS origins to enable CORS with them
    - RATE_LIMIT: to provide rate limiter. MUST BE defined followed config
    RATE_LIMIT_TIME_SPAN and RATE_LIMIT_BLOCK_DURATION
"""
import pydantic
from fastapi import FastAPI
from piccolo_api.csrf.middleware import CSRFMiddleware
from piccolo_api.rate_limiting.middleware import RateLimitingMiddleware, \
    InMemoryLimitProvider
from starlette.middleware.cors import CORSMiddleware


def init_app(app: FastAPI, settings: pydantic.BaseSettings) -> None:
    init_csrf(app, settings)
    init_cors(app, settings)
    init_rate_limiter(app, settings)


def init_csrf(app: FastAPI, settings: pydantic.BaseSettings) -> None:
    if getattr(settings, "SECURITY_CSRF", None):
        app.add_middleware(
            CSRFMiddleware, allowed_hosts=[getattr(settings, "SERVER_HOST")]
        )


def init_cors(app: FastAPI, settings: pydantic.BaseSettings) -> None:
    if getattr(settings, "BACKEND_CORS_ORIGINS", None):
        app.add_middleware(
            CORSMiddleware,
            allow_origins=[str(origin) for origin in
                           getattr(settings, "BACKEND_CORS_ORIGINS")],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )


def init_rate_limiter(app: FastAPI, settings: pydantic.BaseSettings) -> None:
    if getattr(settings, "RATE_LIMIT", None):
        provider = InMemoryLimitProvider(
            limit=getattr(settings, "RATE_LIMIT"),
            timespan=getattr(settings, "RATE_LIMIT_TIME_SPAN"),
            block_duration=getattr(settings, "RATE_LIMIT_BLOCK_DURATION")
        )
        app.add_middleware(
            RateLimitingMiddleware, provider=provider
        )



from typing import Any

import sentry_sdk
from fastapi import FastAPI
from sentry_sdk.integrations.asgi import SentryAsgiMiddleware
from sentry_sdk.integrations.sqlalchemy import SqlalchemyIntegration
from starlette_prometheus import PrometheusMiddleware, metrics


def init_app(app: FastAPI, settings: Any):
    init_sentry(app, settings)
    init_prometheus(app, settings)


def init_sentry(app: FastAPI, settings: Any) -> None:
    if getattr(settings, "SENTRY_DSN", None):
        sentry_sdk.init(dsn=settings.SENTRY_DSN,
                        release=settings.SERVER_VERSION,
                        server_name=settings.SERVER_NAME,
                        environment=settings.SERVER_ENV,
                        in_app_include=settings.SENTRY_INCLUDE,
                        traces_sample_rate=settings.SENTRY_SAMPLE_RATE,
                        integrations=[SqlalchemyIntegration()])
        app.add_middleware(SentryAsgiMiddleware)


def init_prometheus(app: FastAPI, settings: Any):
    if getattr(settings, "PROMETHEUS_ENABLE", None):
        app.add_middleware(PrometheusMiddleware)
        app.add_route(settings.PROMETHEUS_PATH, metrics)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zogutils', 'zogutils.middlewares']

package_data = \
{'': ['*']}

install_requires = \
['fastapi==0.61.1',
 'piccolo-api==0.11.0',
 'sentry-sdk==0.19.0',
 'starlette-prometheus==0.7.0',
 'translation>=1.0.5,<2.0.0']

setup_kwargs = {
    'name': 'zogutils',
    'version': '1.2.0',
    'description': "General utilities to be use in my base-fastapi template. Why ZOG? It's sound like joke and look like zoo.",
    'long_description': '# ZOG Utils\n\nUtilities to use in my base-api project template. https://github.com/tienhm0202/base-fastapi/\n\nWhy ZOG? Because I can\'t named it as `utils` only, so I have to add a prefix.\nZOG sounds like `joke` and looks like `zoo`. I found that funny enough to use.\n\n# Usage\n\n```bash\n$ pip install zogutils\n```\n\n## To generate short unique id string\n\n```python\nfrom zogutils import secret\n\nsecret.unique_id(8, "ID_")\n# return: ID_a7uFg9k0\n```\n\n## To shorten package name like Java\'s Logback\n\n```python\nfrom zogutils import package_name\n\npackage_name.shorten("company.scope.modules.Function", 9)\n# return: (something like) c.s.m.Function - depends on max length\n```\n\n## To init some middlewares\n\n```python\nfrom zogutils import middlewares\nfrom your.app import settings, fastapi_app\n\nmiddlewares.init_app(fastapi_app, settings)\n```\n\n### Configs:\n\n```\n# Sentry\nSENTRY_DSN: Optional[HttpUrl] = None\nSENTRY_INCLUDE: Optional[List[str]] = ["src"]\nSENTRY_SAMPLE_RATE: Optional[float] = 0.5\n\n# CSRF\nSECURITY_CSRF: bool = False\n\n# Rate limit\nRATE_LIMIT: int = 100\nRATE_LIMIT_TIME_SPAN: int = 30\nRATE_LIMIT_BLOCK_DURATION: int = 300\n\n# Prometheus\nPROMETHEUS_ENABLE: bool = True\nPROMETHEUS_PATH: str = "/metrics/"\n\n# Cors\nBACKEND_CORS_ORIGINS: List[AnyHttpUrl] = []\n```',
    'author': 'Hoang Manh Tien',
    'author_email': 'tienhm.0202@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tienhm0202/zogutils',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.0,<3.8.0',
}


setup(**setup_kwargs)
